<?php

/*
Plugin Name: GO.PL Plugin SSL
Plugin URI: https://www.go.pl
Description: Chcesz uruchomić remarketing bez skomplikowanego wdrożenia? Wystarczy, że zainstalujesz ten plugin, w Ustawienia -> Kod GO.PL podasz ID z konta założonego na www.go.pl i możesz emitować reklamy do odwiedzających Twoją stronę.
Version: 1.1
Author: Wojciech Gorczyca
Author URI: https://codeme.eu/
*/

class GoAdService
{

	protected $code;

	protected $provider;

	protected $order;


	public function __construct()
	{
		$this->code = $this->fetchCode();
	}

	public function fetchCode()
	{
		$option = get_option('go_ad_service_option');

		if (is_array($option) && array_key_exists('go_ad_service_code', $option))
		{
			return $option['go_ad_service_code'];
		}

		return $option;
	}

	public function setProvider(string $provider)
	{
		$this->provider = $provider;
	}

	public function attachHooks()
	{
		$this->attachUserHooks();
		
		if(is_admin())
		{
			$this->attachAdminHooks();
		}
	}

	public function attachUserHooks()
	{
		add_action('wp_footer', [$this, 'includeScripts'], 200 );
	}

	public function attachAdminHooks()
	{
		add_action( 'admin_menu', [$this, 'addPluginPage' ]);
		add_action( 'admin_init', [$this, 'initSettingsPage']);
	}

	public function addPluginPage()
	{
		add_options_page(
			'Kod GO.PL',
			'Kod GO.PL',
			'manage_options',
			'go_ad_service_settings',
			[$this, 'createAdminPage']
		);
	}

	public function createAdminPage()
	{
		return require "templates/admin-page.phtml";
	}

	public function initSettingsPage()
	{
		$this->registerSettings();
		$this->addSettings();
	}

	public function registerSettings()
	{
		register_setting(
			'go_ad_service_group', 
			'go_ad_service_option',
			[$this, 'saveCode']
		);
	}

	public function addSettings()
	{
		add_settings_section(
			'go_ad_service_section_campaign',
			'Kampania', 
			[$this, 'printSectionInfo'],
			'go_ad_service_settings' 
		);

		add_settings_field(
			'go_ad_service_code',
			'Kod kampanii', 
			[$this, 'printInputField'],
			'go_ad_service_settings',
			'go_ad_service_section_campaign'
		);
	}

	public function saveCode($input)
    {
        $code = trim($input['go_ad_service_code']);

        if ($this->validateCode($code)) {
            return $input;
        }

        $this->showError();
    }

	public function validateCode($code)
	{
		preg_match("/\w{8}-\w{4}-\w{4}-\w{4}-\w{12}$/", $code, $matches);

		return count($matches[0]) > 0 ? true : false;
	}

	public function showError()
	{
		add_settings_error(
        	'go_ad_service_code_error',
        	esc_attr( 'settings_updated' ),
        	'Błąd! Podany kod ma niewłaściwy format. Upewnij się, że kod ma format xxx-xxxxx-xxxxxxx-xxx-xx',
        	'error'
    	);
	}

	public function printSectionInfo()
	{
		return require 'templates/section-info.phtml';
	}

	public function includeScripts()
	{
		if(null == $this->code){
			return;
		}

		if (is_product()){
			$this->includeProductScript();
		}

		if (is_cart()){
			$this->includeCartScript();
		}

		if(is_wc_endpoint_url('order-received')){
			$this->includeOrderReceivedScript();
		}

		$this->includeCodeScript($this->provider, $this->code);
	}

	public function includeCodeScript($provider, $code)
	{
		return require 'templates/code-script.phtml';
	}

	public function includeProductScript()
	{
		global $product;

		$data = [
			'identifier' => $product->get_id(),
			'availability' => $product->is_in_stock()
		];

		$this->includePushScript('_PRODUCT', json_encode($data));
	}

	public function includeCartScript()
	{
		global $woocommerce;

		$items = $woocommerce->cart->cart_contents;

		$data = array_map(function($productUID, $productData){
			return [
				'identifier' => $productData['product_id'],
				'quantity' => $productData['quantity']
			];
		}, array_keys($items), $items);

		$this->includePushScript('_BASKET', json_encode($data));
	}

	public function includeOrderReceivedScript()
	{

		$this->fetchOrder();

		$items = $this->order->get_items();

		$data = array_map(function($productUID, $productData){
			return [
				'identifier' => (int) $productData['product_id'],
				'quantity' => (int) $productData['qty']
			];
		}, array_keys($items), $items);

		$additional_details = [
			'identifier' => (int) $this->order->id,
			// get_subtotal(do not include cupons discount), get_total (include cupons discount)
			'amount' => (float) $this->order->get_subtotal()
		];

		$this->includePushScript('_ORDER', json_encode($data), json_encode($additional_details));
	}

	public function fetchOrder()
	{
		$uri = $_SERVER['REQUEST_URI'];

		//$id = explode('/', $uri)[3];
		if (preg_match('/order-received\/(.+?)\//',$uri,$t)) {
			
			$id=$t[1];
		}

		$this->order = new WC_Order($id);
	}

	public function includePushScript(string $page, string $data, string $additional_details = '')
	{
		return require 'templates/push-script.phtml';
	}

	public function printInputField()
	{
		$code = $this->code;

		return require 'templates/input-field.phtml';
	}
}


$service = new GoAdService();
$service->setProvider('https://t.goadservices.com/tags/');
$service->attachHooks();



